SystemOrganization addCategory: #'PathObjects-Benchmarks'!

!SProject methodsFor: '*PathObjects-Benchmarks' stamp: 'ls 10/19/2013 11:20:09.842'!
numberOfNonTestClasses
	
	^ self allClassesWithoutTests size! !

!SProject methodsFor: '*PathObjects-Benchmarks' stamp: 'ls 10/19/2013 11:26:42.407'!
numberOfNonTestMethods
	
	^ self allClassesWithoutTests detectSum: [:c | c methodDict size]! !

!SProject methodsFor: '*PathObjects-Benchmarks' stamp: 'ls 10/19/2013 11:24:54.9'!
numberOfTestClasses
	
	^ self testClasses size! !

!SProject methodsFor: '*PathObjects-Benchmarks' stamp: 'ls 10/19/2013 11:24:46.677'!
numberOfTestMethods
	
	^ self allTests size! !

!SProject methodsFor: '*PathObjects-Benchmarks' stamp: 'ls 10/19/2013 11:38:01.5'!
tabular
	
	| result |
	result := WriteStream on: String new.
	result
		nextPutAll: self projectName; tab;
		nextPut: $&; tab;
		nextPutAll: self numberOfNonTestClasses asString; tab;
		nextPut: $&; tab;
		nextPutAll: self numberOfTestClasses asString; tab;
		nextPut: $&; tab;
		nextPutAll: self numberOfNonTestMethods asString; tab;
		nextPut: $&; tab;
		nextPutAll: self numberOfTestMethods asString; tab;
		nextPutAll: '\\'.
	
	^ result contents! !

Object subclass: #SPOBenchmark
	instanceVariableNames: 'testCases projectName results'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PathObjects-Benchmarks'!

!SPOBenchmark class methodsFor: 'accessing' stamp: 'ls 9/16/2013 20:48:55.375'!
blacklist
	
	^ {
		SUnitTest -> #testTestTimeoutLoop. "infinite loop, probably bug in PathTools"
		DTRClassesSpellCheckerTest -> #testCheckSpellingInMethodsOfClassesWithErrors. "Image Freeze"
		
		"The following tests yield GraphViz timeouts but don't break the benchmark"
		WAFileHandlerTest -> #testHandleDoesNotExist.
		DTRCodeSpellCheckerTest -> #testCheckSpellingOfRegression2.
		DTRCorrectionWindowTest -> #testAddToDictionary.
		DTRCorrectionWindowTest -> #testNext.
		DTRPluggableTextMorphPlusTest -> #testThesaurusMenuItems.
		DTRPluggableTextMorphPlusTest -> #testThesaurusMoreSynonyms.
	}! !

!SPOBenchmark class methodsFor: 'instance creation' stamp: 'ls 10/18/2013 12:00:57.592'!
onCurrentSProject
	
	^ self new
		testCases: SProject current allTests;
		projectName: SProject current projectName! !

!SPOBenchmark class methodsFor: 'as yet unclassified' stamp: 'ls 10/18/2013 12:45:18.943'!
spaceConsumption
	
	| currentProject projects result |
	projects := #('Seaside-Core' DicThesaurusRex SUnit SystemBrowser).
	currentProject := SProject current projectName asSymbol.
	result := WriteStream on: String new.
	result nextPutAll: '# project objects messages pointers'; cr.
	
	projects do: [:each | | benchmark |
		15 timesRepeat: [Smalltalk garbageCollect].
		SProject setCurrent: each.
		benchmark := self onCurrentSProject run.
		result nextPutAll: benchmark spaceEvaluation rData].
	
	SProject setCurrent: currentProject.
	^ result contents! !

!SPOBenchmark methodsFor: 'accessing' stamp: 'ls 7/16/2013 13:32:30.891'!
counters
	
	^  self results
		at: #counters
		ifAbsentPut: OrderedCollection new! !

!SPOBenchmark methodsFor: 'accessing' stamp: 'ls 7/15/2013 23:06:53.514'!
errors
	
	^  self results
		at: #errors
		ifAbsentPut: OrderedCollection new! !

!SPOBenchmark methodsFor: 'as yet unclassified' stamp: 'ls 7/25/2013 14:33:52.288'!
killDot
	
	"powershell -Command ""kill -Name Notepad2"""
	Win32Shell new
		shellExecute: nil
		lpOperation: 'open'
		lpFile: 'powershell'
		lpParameters: '-Command "kill -Name dot"'
		lpDirectory: nil
		nShowCmd: 0.! !

!SPOBenchmark methodsFor: 'accessing' stamp: 'ls 10/18/2013 12:01:20.328'!
projectName
	
	^ projectName! !

!SPOBenchmark methodsFor: 'accessing' stamp: 'ls 10/18/2013 12:01:13.471'!
projectName: aString
	
	projectName := aString.! !

!SPOBenchmark methodsFor: 'accessing' stamp: 'ls 7/15/2013 23:04:45.144'!
results
	
	^  results ifNil: [results := Dictionary new]! !

!SPOBenchmark methodsFor: 'running' stamp: 'ls 10/18/2013 15:04:44.822'!
run
	
	| tests |
	tests := self testCases select: [:each | 
		(self class blacklist includes: (each actualClass -> each methodSymbol)) not].
	
	tests do: [:each | | window time |
			window := nil.	
			time := 
				[[window := SPOPathObjects 
					openOn: SProject current 
					class: each actualClass 
					selector: each methodSymbol.
				] timeToRunWithoutGC] on: Error do: [:error |
					self killDot.
					self errors add: (each -> error).
					Transcript show: each asString, ' errored'; cr.
					SPOSystemWindow allInstances do: [:w | w delete]].
		
			window ifNotNilDo: [:w | | model |
				model := (w findA: SPOPathObjects) interactionModel.
				w delete.
				self runtimes add: (each asString -> time).
				self counters add: (each asString -> {
					model steps size.
					model objects size.
					(model steps collect: [:s | s allInvolvedObjects size])
						inject: 0 into: [:a :c | a + c].
				}).
				"Transcript show: each asString, ' time: ', time, 'ms'; cr"].
		
			3 timesRepeat: [Smalltalk garbageCollect]]
		
		displayingProgress: [:each | 'Running ', each].! !

!SPOBenchmark methodsFor: 'evaluations' stamp: 'ls 7/15/2013 23:14:07.367'!
runtimeEvaluation
	
	^  SPORuntimeEvaluation on: self! !

!SPOBenchmark methodsFor: 'accessing' stamp: 'ls 7/15/2013 23:05:25.191'!
runtimes
	
	^  self results
		at: #runtimes
		ifAbsentPut: OrderedCollection new! !

!SPOBenchmark methodsFor: 'evaluations' stamp: 'ls 7/16/2013 14:01:05.302'!
spaceEvaluation
	
	^  SPOSpaceEvaluation on: self! !

!SPOBenchmark methodsFor: 'accessing' stamp: 'ls 7/10/2013 11:42:20.244'!
testCases
	
	^ testCases! !

!SPOBenchmark methodsFor: 'accessing' stamp: 'ls 7/10/2013 11:42:23.322'!
testCases: anObject
	
	testCases := anObject.! !

Object subclass: #SPOBenchmarkEvaluation
	instanceVariableNames: 'benchmark'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PathObjects-Benchmarks'!

!SPOBenchmarkEvaluation class methodsFor: 'instance creation' stamp: 'ls 7/15/2013 23:09:30.608'!
on: aBenchmark
	
	^  self new benchmark: aBenchmark! !

!SPOBenchmarkEvaluation methodsFor: 'accessing' stamp: 'ls 7/15/2013 23:08:54.289'!
benchmark
	
	^ benchmark! !

!SPOBenchmarkEvaluation methodsFor: 'accessing' stamp: 'ls 7/15/2013 23:08:57.193'!
benchmark: anObject
	
	benchmark := anObject.! !

!SPOBenchmarkEvaluation methodsFor: 'accessing' stamp: 'ls 10/18/2013 12:01:41.048'!
projectName
	
	^ self benchmark projectName! !

SPOBenchmarkEvaluation subclass: #SPORuntimeEvaluation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PathObjects-Benchmarks'!

!SPORuntimeEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 9/18/2013 20:13:55.118'!
gnuplotData
	
	| h total cumulative result |
	h := self histogram.
	total := h runs inject: 0 into: [:x :n | x + n].
	cumulative := 0.
	
	result := WriteStream on: String new.
	result
		nextPutAll: '# Bucket (ms)';
		tab;
		nextPutAll: 'Occurrences';
		tab;
		nextPutAll: '%';
		tab;
		nextPutAll: 'Cum. %';
		cr.
	
	self histogram runsAndValuesDo: [:run :value | | p |
		p := ((run / total) * 100) truncateTo: 0.01.
		cumulative := cumulative + p.
		result
			nextPutAll: value asString;
			tab;
			nextPutAll: run asString;
			tab;
			nextPutAll: p asString;
			tab;
			nextPutAll: cumulative asString;
			cr].
	
	^  result contents! !

!SPORuntimeEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 9/8/2013 20:37:17.247'!
histogram
	
	| times |
	times := (self benchmark runtimes collect: [:each | each value roundUpTo: 1000]) sort.
	^ RunArray newFrom: times! !

!SPORuntimeEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 9/8/2013 20:37:27.102'!
histogrammCsv
	
	| result |
	result := WriteStream on: String new.
	self histogram runsAndValuesDo: [:run :value |
		result
			nextPutAll: (value / 1000) asString;
			nextPut: $,;
			nextPutAll: run asString;
			cr].
	^  result contents! !

!SPORuntimeEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 10/16/2013 18:17:48.219'!
runtimes
	
	| times result |
	times := (self benchmark runtimes collect: [:each | each value asFloat]) sort.
	result := WriteStream on: String new.
	
	result
		nextPutAll: '# runtime (ms)';
		cr.
	times do: [:each | result nextPutAll: each asString; cr].
	^ result contents! !

!SPORuntimeEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 7/16/2013 11:36:36.828'!
simpleStatistics
	
	| times |
	times := (self benchmark runtimes collect: [:each | each value asFloat]) sort.
	^ Dictionary newFrom: {
		#mean -> (times average).
		#median -> (times median).
		#minMax -> (times minMax).
	}! !

SPOBenchmarkEvaluation subclass: #SPOSpaceEvaluation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PathObjects-Benchmarks'!

!SPOSpaceEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 7/16/2013 14:59:07.832'!
messageStats
	
	| objectCounts |
	objectCounts := self benchmark counters collect: [:each | each value first asFloat].
	^ Dictionary newFrom: {
		#mean -> (objectCounts average).
		#median -> (objectCounts median).
		#minMax -> (objectCounts minMax).
	}! !

!SPOSpaceEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 7/16/2013 14:59:01.027'!
objectStats
	
	| objectCounts |
	objectCounts := self benchmark counters collect: [:each | each value second asFloat].
	^ Dictionary newFrom: {
		#mean -> (objectCounts average).
		#median -> (objectCounts median).
		#minMax -> (objectCounts minMax).
	}! !

!SPOSpaceEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 7/16/2013 14:58:56.188'!
pointerStats
	
	| objectCounts |
	objectCounts := self benchmark counters collect: [:each | each value third asFloat].
	^ Dictionary newFrom: {
		#mean -> (objectCounts average).
		#median -> (objectCounts median).
		#minMax -> (objectCounts minMax).
	}! !

!SPOSpaceEvaluation methodsFor: 'as yet unclassified' stamp: 'ls 10/18/2013 12:04:09.776'!
rData
	"format: <projectName> <objectCount> <messageCount> <pointerCount>"
	
	| result |
	result := WriteStream on: String new.
	self benchmark counters do: [:each | | objects messages pointers |
		objects := each value second.
		messages := each value first.
		pointers := each value third.
		
		result
			nextPutAll: self projectName; tab;
			nextPutAll: objects asString; tab;
			nextPutAll: messages asString; tab;
			nextPutAll: pointers asString; cr].
		
	^ result contents! !

Object subclass: #SPOTimestampMutator
	instanceVariableNames: 'methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PathObjects-Benchmarks'!

!SPOTimestampMutator class methodsFor: 'instance creation' stamp: 'ls 12/4/2013 22:33:32.206'!
onCurrentSProject
	
	| methods |
	methods := SProject current allMethods.
	
	^ self new
		methods: methods;
		yourself! !

!SPOTimestampMutator methodsFor: 'as yet unclassified' stamp: 'ls 10/24/2013 12:08:31.384'!
apply
	
	self methods do: [:each | each actualClass
		compile: each source
		classified: each category
		withStamp: self randomChangeStamp
		notifying: nil].! !

!SPOTimestampMutator methodsFor: 'accessing' stamp: 'ls 10/24/2013 11:44:10.02'!
methods
	
	^ methods ifNil: [methods := OrderedCollection new]
! !

!SPOTimestampMutator methodsFor: 'accessing' stamp: 'ls 10/24/2013 11:43:57.854'!
methods: anObject
	
	methods := anObject.
! !

!SPOTimestampMutator methodsFor: 'as yet unclassified' stamp: 'ls 10/24/2013 11:46:05.777'!
randomAuthor
	
	^  (String new: 3) collect: [:each | 'abcdefghijklmnopqrstuvwxyz' atRandom]! !

!SPOTimestampMutator methodsFor: 'as yet unclassified' stamp: 'ls 10/24/2013 12:08:20.31'!
randomChangeStamp
	
	| randomDateTime |
	randomDateTime := self randomDateAndTime.
	^ self randomAuthor , ' ' , randomDateTime asDate mmddyyyy, ' ',
		((String streamContents: [:s | randomDateTime asTime print24: true on: s]))! !

!SPOTimestampMutator methodsFor: 'as yet unclassified' stamp: 'ls 10/24/2013 12:01:15.231'!
randomDateAndTime
	
	| min max rnd |
	min := 3540067200. "2013-03-07T00:00:00+01:00"
	max := 3550780800. "2013-07-09T00:00:00+01:00"
	rnd := min + ((max - min) atRandom).
	
	^ DateAndTime fromSeconds: rnd! !
